/*
 * id3tag.c -- id3tag handling routine
 * (C)Copyright 1998, 99, 2000 by Hiroshi Takekawa
 * Last Modified: Tue Nov  7 15:28:42 2000.
 * $Id: id3tag.c,v 1.8 2001/03/31 02:31:06 sian Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "id3tag.h"

#ifndef SEEK_SET
#  define SEEK_SET 0
#  define SEEK_CUR 1
#  define SEEK_END 2
#endif

/* genre list */
static char *genrename[] = {
  "Blues",                  "Classic Rock",
  "Country",                "Dance",
  "Disco",                  "Funk",
  "Grunge",                 "Hip-Hop", 
  "Jazz",                   "Metal",
  "New Age",                "Oldies",
  "Other",                  "Pop",
  "R&B",                    "Rap",
  "Reggae",                 "Rock",
  "Techno",                 "Industrial",
  "Alternative",            "Ska",
  "Death Metal",            "Pranks",
  "Soundtrack",             "Euro-Techno", 
  "Ambient",                "Trip-Hop", 
  "Vocal",                  "Jazz+Funk", 
  "Fusion",                 "Trance",
  "Classical",              "Instrumental", 
  "Acid",                   "House",
  "Game",                   "Sound Clip", 
  "Gospel",                 "Noise",
  "Alt. Rock",              "Bass",
  "Soul",                   "Punk",
  "Space",                  "Meditative",
  "Instrumental Pop",       "Instrumental Rock", 
  "Ethnic",                 "Gothic",
  "Darkwave",               "Techno-Industrial", 
  "Electronic",             "Pop-Folk",
  "Eurodance",              "Dream",
  "Southern Rock",          "Comedy", 
  "Cult",                   "Gangsta",
  "Top 40",                 "Christian Rap", 
  "Pop/Funk",               "Jungle",
  "Native American",        "Cabaret",
  "New Wave",               "Psychadelic", 
  "Rave",                   "Showtunes", 
  "Trailer",                "Lo-Fi",
  "Tribal",                 "Acid Punk",
  "Acid Jazz",              "Polka",
  "Retro",                  "Musical",
  "Rock & Roll",            "Hard Rock", 
  "Folk",                   "Folk/Rock",
  "National Folk",          "Swing",
  "Fusion",                 "Bebob",
  "Latin",                  "Revival",
  "Celtic",                 "Bluegrass",
  "Avantgarde",             "Gothic Rock",
  "Progressive Rock",       "Psychedelic Rock", 
  "Symphonic Rock",         "Slow Rock",
  "Big Band",               "Chorus",
  "Easy Listening",         "Acoustic", 
  "Humour",                 "Speech",
  "Chanson",                "Opera",
  "Chamber Music",          "Sonata",
  "Symphony",               "Booty Bass", 
  "Primus",                 "Porn Groove", 
  "Satire",                 "Slow Jam", 
  "Club",                   "Tango",
  "Samba",                  "Folklore",
  "Ballad",                 "Power Ballad",
  "Rhythmic Soul",          "Freestyle",
  "Duet",                   "Punk Rock",
  "Drum Solo",              "A Cappella",
  "Euro-House",             "Dance Hall",
  "Goa",                    "Drum & Bass",
  "Club-House",             "Hardcore",
  "Terror",                 "Indie",
  "BritPop",                "Negerpunk",
  "Polsk Punk",             "Beat",
  "Christian Gangsta Rap",  "Heavy Metal",
  "Black Metal",            "Crossover",
  "Contemporary Christian", "Christian Rock",
  "Merengue",               "Salsa",
  "Thrash Metal",           "Anime",
  "JPop",                   "Synthpop",
  NULL
};

static FILE *fp;
static Id3tag tag;
static int if_force = 0;

int
id3tag_init(char *filename, int writeflag)
{
  tag.c = '\0';
  if (fp != NULL)
    return 2;

  if ((fp = fopen(filename, writeflag ? "r+" : "r")) == NULL)
    return 0;
  return 1;
}

int
id3tag_finish(void)
{
  int f = !fclose(fp);

  fp = NULL;
  return f;
}

int
id3tag_readtag(void)
{
  if (fseek(fp, -128L, SEEK_END))
    return 0;
  return fread(&tag, 128, 1, fp);
}

int
id3tag_writetag(void)
{
  if (memcmp(tag.id, "TAG", 3) == 0)
    fseek(fp, -128L, SEEK_END);
  else {
    memcpy(tag.id, "TAG", 3);
    fseek(fp, 0L, SEEK_END);
  }
  return fwrite(&tag, 128, 1, fp);
}

int
id3tag_check(void)
{
  if (fp == NULL)
    return 0;
  if (memcmp(tag.id, "TAG", 3) != 0) {
    memset(&tag, 0, sizeof(tag));
    return 0;
  }
  return 1;
}

/* return: deleted = 0, no tag = 1, error at deletion = -1 */
int
id3tag_delete(void)
{
  if (memcmp(tag.id, "TAG", 3) == 0) {
    int size;
    fseek(fp, 0L, SEEK_END);
    size = ftell(fp);
    return ftruncate(fileno(fp), size - 128);
  }
  return 1;
}

void
id3tag_set_force_flag(int flag)
{
  if_force = flag;
}

void
trim(char *src, int len)
{
  int i;

  /* 
   * below is bit faster on big endian processor? (e.g. intel 86)
   * for ( i=len-1 ; src[i]&0x80 && i>=0 ; i-- ) ;
   */
  for ( i=len-1 ; (signed char)src[i]<0 && i>=0 ; i-- ) ;
  if ( (len-i)%2 == 0 ) len--;
  src[len] = '\0';
}

static void *
setstring(void *dist, void *src, size_t len)
{
  if (strlen(src) > len) {
    if (!if_force) {
      return NULL;
    } else {
      memcpy(dist, src, len);
      trim(dist,len);
      return dist;
    }
  }

  if (strlen(src) == 0) {
    unsigned char *d = (unsigned char *)dist;
    *d = '\0';
  }

  if (strlen(src) == len)
    return memcpy(dist, src, len);
  else
    return strcpy(dist, src);
}

static void *
getstring(char *src, size_t len)
{
  unsigned char *str;

  if (strlen(src) == 0)
    src = "\0";

  if ((str = malloc(strlen(src) + 1)) == NULL)
    return NULL;

  if (strlen(src) < len)
    strcpy(str, src);
  else {
    memcpy(str, src, len);
    str[len] = '\0';
  }
  return str;
}

unsigned char *
id3tag_getsong(void)
{
  return getstring(tag.song, 30);
}

unsigned char *
id3tag_setsong(unsigned char *song)
{
  return setstring(tag.song, song, 30);
}

unsigned char *
id3tag_getartist(void)
{
  return getstring(tag.artist, 30);
}

unsigned char *
id3tag_setartist(unsigned char *artist)
{
  return setstring(tag.artist, artist, 30);
}

unsigned char *
id3tag_getalbum(void)
{
  return getstring(tag.album, 30);
}

unsigned char *
id3tag_setalbum(unsigned char *album)
{
  return setstring(tag.album, album, 30);
}

char *
id3tag_getyear(void)
{
  return getstring(tag.year, 4);
}

char *
id3tag_setyear(char *year)
{
  return setstring(tag.year, year, 4);
}

unsigned char *
id3tag_getcomment(void)
{
  return getstring(tag.comment, 30);
}

unsigned char *
id3tag_setcomment(unsigned char *comment)
{
  return setstring(tag.comment, comment, 30);
}

unsigned char
id3tag_getgenrecode(void)
{
  return tag.genre;
}

void
id3tag_setgenrecode(unsigned char genre)
{
  tag.genre = genre;
}

char *
id3tag_getgenre(void)
{
  return genrename[(int)id3tag_getgenrecode()];
}

int
id3tag_setgenre(char *genre)
{
  int i = 0;

  if (genre == NULL) {
    fprintf(stderr, "tag.genre = %d\n", tag.genre);
    return tag.genre;
  }
  while (genrename[i] != NULL) {
    if (genrename[i][0] == *genre && (strcmp(genrename[i], genre) == 0)) {
      id3tag_setgenrecode((unsigned char)i);
      return i;
    }
    i++;
  }
  return -1;
}

char **
id3tag_getgenrenames(void)
{
  return genrename;
}

Id3tag
id3tag_getinfo(void)
{
  return tag;
}

void
id3tag_setinfo(Id3tag giventag)
{
  tag = giventag;
}
