/*
 * settag.c -- set id3tag information
 * (C)Copyright 1998, 99, 2000, 2001 by Hiroshi Takekawa
 * Last Modified: Sun Apr  1 20:59:07 2001.
 * $Id: settag.c,v 1.7 2001/04/01 11:59:34 sian Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "id3tag.h"
#include "kanji.h"

#define VERSION "20010401"
#define CONV(s) (kanji_euc2sjis_string(s))

static void
usage(void)
{
  fprintf(stderr, "settag version " VERSION "\n");
  fprintf(stderr, "(C)Copyright 1998, 99, 2000, 2001 by Hiroshi Takekawa\n");
  fprintf(stderr, "usage: settag [options] filename\n");
  fprintf(stderr, "options:\n");
  fprintf(stderr, " -s: song name\n");
  fprintf(stderr, " -a: artist name\n");
  fprintf(stderr, " -A: album name\n");
  fprintf(stderr, " -y: year name\n");
  fprintf(stderr, " -c: comment\n");
  fprintf(stderr, " -g: genre name\n");
  fprintf(stderr, " -G: show genre names\n");
  fprintf(stderr, " -R: remove tag\n");
  fprintf(stderr, " -f: force setting when longer than limit\n");
  fprintf(stderr, " -E: use EUC instead of SJIS\n");
  fprintf(stderr, " -h: show this help\n");
}

static void
showgenre(void)
{
  int i, j, l;
  char **genrename = id3tag_getgenrenames();

  i = j = 0;
  printf("Genre list:\n");
  while (genrename[i] != NULL) {
    l = strlen(genrename[i]);
    printf("%s", genrename[i++]);
    if (j % 4 == 3)
      printf("\n");
    else {
      printf("\t");
      if (l < 8)
	printf("\t");
      if (l >= 16) {
	j++;
	printf(j % 4 == 3 ? "\n" : "\t");
      }
    }
    j++;
  }
  printf("\n");
}

int
main(int argc, char **argv)
{
  extern char *optarg;
  extern int optind; /* opterr, optopt; */
  int c, delete = 0;
  unsigned char *_song = NULL, *_artist = NULL, *_album = NULL, *_comment = NULL;
  unsigned char *song, *artist, *album, *comment;
  char *year, *genre;
  int if_song, if_artist, if_album, if_comment, if_year, if_genre, if_euc, if_force;

  song = artist = album = comment = year = "";
  genre = "Pop";
  if_song = if_artist = if_album = if_comment = if_year = if_genre = 0;
  if_euc = 0;
  /* if_euc = 1; */
  if_force = 0;
  /* if_force = 1; */

  while ((c = getopt(argc, argv, "s:a:A:y:c:g:RGEfh")) != -1) {
    switch (c) {
    case 'h':
      usage();
      exit(0);
      break;
    case 's':
      _song = strdup(optarg);
      if_song = 1;
      break;
    case 'a':
      _artist = strdup(optarg);
      if_artist = 1;
      break;
    case 'A':
      _album = strdup(optarg);
      if_album =1;
      break;
    case 'y':
      year = optarg;
      if_year = 1;
      break;
    case 'c':
      _comment = strdup(optarg);
      if_comment = 1;
      break;
    case 'g':
      genre = optarg;
      if_genre = 1;
      break;
    case 'E':
      if_euc = 1;
      break;
    case 'f':
      if_force = 1 - if_force;
      break;
    case 'G':
      showgenre();
      exit(0);
      break;
    case 'R':
      delete = 1;
      break;
    default:
      usage();
      exit(1);
    }
  }

  if (optind == argc) {
    usage();
    exit(2);
  }

  if (if_euc) {
    song = _song;
    artist = _artist;
    album = _album;
    comment = _comment;
  } else {
    song = CONV(_song);
    artist = CONV(_artist);
    album = CONV(_album);
    comment = CONV(_comment);
    free(_song);
    free(_artist);
    free(_album);
    free(_comment);
  }

  id3tag_set_force_flag(if_force);

  while (optind < argc) {
    if (!id3tag_init(argv[optind], 1)) {
      fprintf(stderr, "Can't open file %s\n", argv[optind]);
      perror(argv[0]);
      exit(3);
      fprintf(stderr, "Whoops!? exit returns?\n");
    }

    printf("Filename: %s ", argv[optind]);

    if (!id3tag_readtag()) {
      fprintf(stderr, "Can't read tag\n");
      exit(4);
    }

    if (delete) {
      int f = id3tag_delete();
      if (f == 0)
	printf("tag deleted.\n");
      else if (f == -1)
	printf("error.\n");
      else
	printf("no tag.\n");
    } else {
      if (!id3tag_check())
	printf("new tag ");

      if (if_song && (id3tag_setsong(song) == NULL)) {
	fprintf(stderr, "Can't set song name\n");
	exit(5);
      }
      if (if_artist && (id3tag_setartist(artist) == NULL)) {
	fprintf(stderr, "Can't set artist name\n");
	exit(5);
      }

      if (if_album && (id3tag_setalbum(album) == NULL)) {
	fprintf(stderr, "Can't set album name\n");
	exit(5);
      }

      if (if_year && (id3tag_setyear(year) == NULL)) {
	fprintf(stderr, "Can't set year\n");
	exit(5);
      }

      if (if_comment && (id3tag_setcomment(comment) == NULL)) {
	fprintf(stderr, "Can't set comment\n");
	exit(5);
      }

      if (if_genre && (id3tag_setgenre(genre) < 0)) {
	fprintf(stderr, "Can't set genre\n");
	exit(4);
      }
      printf("set.\n");
      id3tag_writetag();
    }
    id3tag_finish();
    optind++;
  }
  return 0;
}
