/*
 * showtag.c -- show id3tag information
 * (C)Copyright 1998, 99, 2000, 2001 by Hiroshi Takekawa
 * Last Modified: Sat Mar 31 14:54:43 2001.
 * $Id: showtag.c,v 1.6 2001/03/31 05:55:38 sian Exp $
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <dirent.h>
#include <sys/stat.h>
#include "id3tag.h"
#include "kanji.h"
#include "dlist.h"

#define CONV(s) (kanji_sjis2euc_string(s))

#define VERSION "20010331"

static void
usage(char **argv)
{
  fprintf(stderr, "showtag version %s\n", VERSION);
  fprintf(stderr, "(C)Copyright 1998, 99, 2000, 2001 by Hiroshi Takekawa\n\n");
  fprintf(stderr, "Usage: %s [-cgn] pathname [pathname..]\n", argv[0]);
  fprintf(stderr, " -c: show comment\n");
  fprintf(stderr, " -g: show genre\n");
  fprintf(stderr, " -s: disable path name sorting\n");
  fprintf(stderr, " -n: find no tagged files\n");
}

static void
add_to_dlist(Dlist *dl, char *filename)
{
  struct stat s;
  DIR *d;
  struct dirent *dir;

  if (stat(filename, &s) < 0)
    fprintf(stderr, "Can't stat: %s\n", filename);
  else if (S_ISDIR(s.st_mode)) {
    /* This is a directory... */
    if ((d = opendir(filename)) == NULL) {
      fprintf(stderr, "Can't opendir: %s\n", filename);
      return;
    }
    while ((dir = readdir(d)) != NULL)
      if (strcmp(dir->d_name, ".") && strcmp(dir->d_name, "..")) {
	//char *f = malloc(strlen(filename) + 1 + strlen(dir->d_name) + 1);
	char f[strlen(filename) + 1 + strlen(dir->d_name) + 1];

	if (f == NULL) {
	  fprintf(stderr, "No enough memory\n");
	  exit(-2);
	}
	strcpy(f, filename);
	if (filename[strlen(filename) - 1] != '/')
	  strcat(f, "/");
	strcat(f, dir->d_name);
	add_to_dlist(dl, f);
	//free(f);
      }
    closedir(d);
  } else {
    if (*filename == '.' && *(filename + 1) == '/') {
      filename++;
      while (*filename == '/')
	filename++;
    }
    if (!dlist_add_str(dl, filename)) {
      fprintf(stderr, "Dlist error: (maybe) no enough memory\n");
      exit(-2);
    }
  }
}

int
main(int argc, char **argv)
{
  extern char *optarg;
  extern int optind; /* opterr, optopt; */
  int c;
  unsigned char *song = NULL, *artist = NULL, *album = NULL, *comment = NULL, *genre, *year;
  unsigned char *s1, *s2, *s3, *s4;
  int c1, c2, c3, c4;
  int index = 0, if_comment = 0, if_genre = 0, if_notag = 0, if_sort = 1;
  Dlist *dl;
  Dlist_data *dd;

  while ((c = getopt(argc, argv, "cgnsh")) != -1) {
    switch (c) {
    case 'c':
      if_comment = 1;
      break;
    case 'g':
      if_genre = 1;
      break;
    case 'n':
      if_notag = 1;
      break;
    case 's':
      if_sort = 0;
      break;
    case 'h':
      usage(argv);
      exit(0);
    default:
      usage(argv);
      exit(-1);
    }
  }

  if (optind == argc) {
    usage(argv);
    exit(-1);
  }

  dl = dlist_create();
  while (optind < argc)
    add_to_dlist(dl, argv[optind++]);

  if (if_sort)
    dlist_sort(dl);

  dlist_iter(dl, dd) {
    index++;

    if (!id3tag_init(dlist_data(dd), 0)) {
      fprintf(stderr, "Can't open file %s\n", (char *)dlist_data(dd));
      perror(argv[0]);
      continue;
    }

    if (!id3tag_readtag()) {
      fprintf(stderr, "Can't read tag [%s]\n", (char *)dlist_data(dd));
      id3tag_finish();
      continue;
    }

    if (!id3tag_check()) {
      printf("%3d filename: %s\n", index, (char *)dlist_data(dd));
      printf("No tag.\n");
      id3tag_finish();
      continue;
    }

    s1 = id3tag_getsong();
    s2 = id3tag_getartist();
    s3 = id3tag_getalbum();
    s4 = id3tag_getcomment();
    kanji_initialize(); (void)kanji_checkcode_string(s1, &c1);
    kanji_initialize(); (void)kanji_checkcode_string(s2, &c2);
    kanji_initialize(); (void)kanji_checkcode_string(s3, &c3);
    kanji_initialize(); (void)kanji_checkcode_string(s4, &c4);
    if (c1 == SJIS) {
      song = CONV(s1);
      free(s1);
    } else {
      song = s1;
    }
    if (c2 == SJIS) {
      artist = CONV(s2);
      free(s2);
    } else {
      artist = s2;
    }
    if (c3 == SJIS) {
      album = CONV(s3);
      free(s3);
    } else {
      album = s3;
    }
    if (c4 == SJIS) {
      comment = CONV(s4);
      free(s4);
    } else {
      comment = s4;
    }

    genre = id3tag_getgenre();
    year = id3tag_getyear();

    if (!if_notag) {
      printf("%3d filename: %s\n", index, (char *)dlist_data(dd));
      printf("̾: %s\n", song);
      printf("ƥ: %s\n", artist);
      printf("Х: %s(%s)\n", album, year);
      if (if_genre)
	printf(": %s\n", genre);
      if (if_comment)
	printf(": %s\n", comment);
    }

    free(song);
    free(artist);
    free(album);
    free(comment);
    free(year);
    id3tag_finish();
  }

  return 0;
}
